@@ -8,6 +8,9 @@
     revision             : $Id: tqslpaths.h,v 1.7 2013/03/01 13:09:28 k1mu Exp $
  ***************************************************************************/
 
+#ifndef __tqslpaths_h
+#define __tqslpaths_h
+
 #ifdef HAVE_CONFIG_H
 #include "sysconfig.h"
 #endif
@@ -22,8 +25,8 @@
 #endif
 
 class DocPaths : public wxPathList {
-public:
-	DocPaths(wxString subdir) : wxPathList() {
+ public:
+	explicit DocPaths(wxString subdir) : wxPathList() {
 #ifdef _WIN32
 		Add(wxGetHomeDir() + wxT("\\help\\") + subdir);
 #else
@@ -33,12 +36,12 @@ public:
 		HKEY hkey;
 		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\TrustedQSL",
 			0, KEY_READ, &hkey) == ERROR_SUCCESS) {
-
 			DWORD dtype;
 			char path[256];
 			DWORD bsize = sizeof path;
 			if (RegQueryValueEx(hkey, "HelpDir", 0, &dtype, (LPBYTE)path, &bsize)
 				== ERROR_SUCCESS) {
+				Add(wxString(path) + "\\");
 				Add(wxString(path) + "\\" + subdir);
 			}
 		}
@@ -56,29 +59,32 @@ public:
 			CFIndex lenB = CFStringGetMaximumSizeForEncoding(CFStringGetLength(bundleString), kCFStringEncodingUTF8);
 			CFIndex lenR = CFStringGetMaximumSizeForEncoding(CFStringGetLength(resString), kCFStringEncodingUTF8);
 
-			char *npath = (char *) malloc(lenB  + lenR + 10);
+			char *npath = reinterpret_cast<char *>(malloc(lenB  + lenR + 10));
 			if (npath) {
 				CFStringGetCString(bundleString, npath, lenB + 1, kCFStringEncodingASCII);
 				CFRelease(bundleString);
 
                 		// if last char is not a /, append one
                 		if ((strlen(npath) > 0) && (npath[strlen(npath)-1] != '/')) {
-                        		strcat(npath,"/");
+                        		strncat(npath, "/", 2);
                 		}
 
 				CFStringGetCString(resString, npath + strlen(npath), lenR + 1, kCFStringEncodingUTF8);
 
                 		if ((strlen(npath) > 0) && (npath[strlen(npath)-1] != '/')) {
-                        		strcat(npath,"/");
+                        		strncat(npath, "/", 2);
                 		}
-                		Add(wxString(npath, wxConvLocal) + wxT("Help/"));
+                		Add(wxString::FromUTF8(npath) + wxT("Help/"));
+				Add(wxT("/Applications/") + subdir + wxT(".app/Contents/Resources/"));
 				Add(wxT("/Applications/") + subdir + wxT(".app/Contents/Resources/Help/"));
-				free (npath);
+				free(npath);
 			}
 		}
 #else
+		Add(wxT(CONFDIR) wxT("help/"));
 		Add(wxT(CONFDIR) wxT("help/") + subdir);
 		Add(subdir);
 #endif
 	}
 };
+#endif		// tqslpaths_h
