@@ -37,7 +37,7 @@ TQSLValidator::TransferFromWindow() {
 		return FALSE;
 	if (_objp == 0)
 		return FALSE;
-	wxTextCtrl *ctl = (wxTextCtrl *) m_validatorWindow;
+	wxTextCtrl *ctl = reinterpret_cast<wxTextCtrl *>(m_validatorWindow);
 	wxString str = ctl->GetValue();
 	FromString(str);
 	return TRUE;
@@ -53,26 +53,26 @@ TQSLValidator::TransferToWindow() {
 	if (_objp == 0)
 		return FALSE;
 	wxString str = this->ToString();
-	wxTextCtrl *ctl = (wxTextCtrl *) m_validatorWindow;
+	wxTextCtrl *ctl = reinterpret_cast<wxTextCtrl *>(m_validatorWindow);
 	ctl->SetValue(str);
 	return TRUE;
 }
 
 bool
 TQSLValidator::Validate(wxWindow* parent) {
-	tqslTrace("TQSLValidator::Validate", "parent=%lx", (void *)parent);
+	tqslTrace("TQSLValidator::Validate", "parent=%lx", reinterpret_cast<void *>(parent));
 	if (!m_validatorWindow)
 		return FALSE;
 	if (!m_validatorWindow->IsKindOf(CLASSINFO(wxTextCtrl)))
 		return FALSE;
 	if (_objp == 0)
 		return FALSE;
-	wxTextCtrl *ctl = (wxTextCtrl *) m_validatorWindow;
+	wxTextCtrl *ctl = reinterpret_cast<wxTextCtrl *>(m_validatorWindow);
 	wxString str = ctl->GetValue();
 	if (!IsValid(str)) {
 		m_validatorWindow->SetFocus();
 		wxString buf;
-		buf.Printf(wxT("Invalid %s: \"%s\""), (const char *)_type.mb_str(), (const char *)str.mb_str());
+		buf.Printf(wxT("Invalid %s: \"%s\""), (const char *)_type.ToUTF8(), (const char *)str.ToUTF8());
 		wxMessageBox(buf, wxT("QSO Data Error"), wxOK | wxICON_EXCLAMATION, parent);
 		return FALSE;
 	}
@@ -82,50 +82,50 @@ TQSLValidator::Validate(wxWindow* parent) {
 void
 TQSLDateValidator::FromString(const wxString& str) {
 	if (_objp != 0)
-		tqsl_initDate((tQSL_Date *)_objp, str.mb_str());
+		tqsl_initDate(reinterpret_cast<tQSL_Date *>(_objp), str.ToUTF8());
 }
 
 wxString
 TQSLDateValidator::ToString() {
 	if (_objp == 0)
 		return wxT("");
-	tQSL_Date *_datep = (tQSL_Date *)_objp;
+	tQSL_Date *_datep = reinterpret_cast<tQSL_Date *>(_objp);
 	if (!tqsl_isDateValid(_datep))
 		return wxT("");
 	char buf[20];
 	tqsl_convertDateToText(_datep, buf, sizeof buf);
-	return wxString(buf, wxConvLocal);
+	return wxString::FromUTF8(buf);
 }
 
 bool
 TQSLDateValidator::IsValid(const wxString& str) {
-	tqslTrace("TQSLDateValidator::IsValid", "str=%s", _S(str));
+	tqslTrace("TQSLDateValidator::IsValid", "str=%s", S(str));
 	tQSL_Date d;
-	return (!tqsl_initDate(&d, str.mb_str()) && tqsl_isDateValid(&d));
+	return (!tqsl_initDate(&d, str.ToUTF8()) && tqsl_isDateValid(&d));
 }
 
 void
 TQSLTimeValidator::FromString(const wxString& str) {
-	tqslTrace("TQSLTimeValidator::FromString", "str=%s", _S( str));
+	tqslTrace("TQSLTimeValidator::FromString", "str=%s", S(str));
 	if (_objp != 0)
-		tqsl_initTime((tQSL_Time *)_objp, str.mb_str());
+		tqsl_initTime(reinterpret_cast<tQSL_Time *>(_objp), str.ToUTF8());
 }
 
 wxString
 TQSLTimeValidator::ToString() {
 	if (_objp == 0)
 		return wxT("");
-	tQSL_Time *_timep = (tQSL_Time *)_objp;
+	tQSL_Time *_timep = reinterpret_cast<tQSL_Time *>(_objp);
 	if (!tqsl_isTimeValid(_timep))
 		return wxT("");
 	char buf[20];
 	tqsl_convertTimeToText(_timep, buf, sizeof buf);
-	return wxString(buf, wxConvLocal);
+	return wxString::FromUTF8(buf);
 }
 
 bool
 TQSLTimeValidator::IsValid(const wxString& str) {
-	tqslTrace("TQSLTimeValidator::IsValid", "str=%s", _S(str));
+	tqslTrace("TQSLTimeValidator::IsValid", "str=%s", S(str));
 	tQSL_Time t;
-	return (!tqsl_initTime(&t, str.mb_str()) && tqsl_isTimeValid(&t));
+	return (!tqsl_initTime(&t, str.ToUTF8()) && tqsl_isTimeValid(&t));
 }
