@@ -1152,6 +1152,9 @@ update_page(int page, TQSL_LOCATION *loc) {
 						field.idx = static_cast<int>(field.items.size());
 					field.items.push_back(item);
 				}
+				if (field.idx == -1 && field.items.size() == 1) {
+					field.idx = 0;
+				}
 				if (field.idx >= 0) {
 					field.cdata = field.items[field.idx].text;
 				}
