@@ -20,7 +20,6 @@
 #include "tqslerrno.h"
 #include <cstring>
 #include <string>
-#include <vector>
 #include <ctype.h>
 #include <set>
 #include <db.h>
@@ -31,19 +30,22 @@
 #ifndef _WIN32
     #include <unistd.h>
     #include <dirent.h>
+#else
+    #include <direct.h>
+    #include "windirent.h"
 #endif
 
 #include "winstrdefs.h"
 
-using namespace std;
-
+using std::set;
+using std::string;
 
 static bool checkCallSign(const string& call);
 
 namespace tqsllib {
 
 class TQSL_CONVERTER {
-public:
+ public:
 	TQSL_CONVERTER();
 	~TQSL_CONVERTER();
 	void clearRec();
@@ -160,9 +162,7 @@ inline void TQSL_CONVERTER::clearRec() {
 
 }	// namespace tqsllib
 
-using namespace tqsllib;
-
-
+using tqsllib::TQSL_CONVERTER;
 
 static char *
 tqsl_strtoupper(char *str) {
@@ -181,17 +181,17 @@ check_conv(tQSL_Converter conv) {
 }
 
 static tqsl_adifFieldDefinitions adif_qso_record_fields[] = {
-	{ "CALL", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_CALLSIGN_MAX, 0, 0, NULL },
-	{ "BAND", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_BAND_MAX, 0, 0, NULL },
-	{ "MODE", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_MODE_MAX, 0, 0, NULL },
-	{ "QSO_DATE", "", TQSL_ADIF_RANGE_TYPE_NONE, 10, 0, 0, NULL },
-	{ "TIME_ON", "", TQSL_ADIF_RANGE_TYPE_NONE, 10, 0, 0, NULL },
-	{ "FREQ", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_FREQ_MAX, 0, 0, NULL },
-	{ "FREQ_RX", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_FREQ_MAX, 0, 0, NULL },
-	{ "BAND_RX", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_BAND_MAX, 0, 0, NULL },
-	{ "SAT_NAME", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_SATNAME_MAX, 0, 0, NULL },
-	{ "PROP_MODE", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_PROPMODE_MAX, 0, 0, NULL },
-	{ "eor", "", TQSL_ADIF_RANGE_TYPE_NONE, 0, 0, 0, NULL },
+        { "CALL", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_CALLSIGN_MAX, 0, 0, NULL },
+        { "BAND", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_BAND_MAX, 0, 0, NULL },
+        { "MODE", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_MODE_MAX, 0, 0, NULL },
+        { "QSO_DATE", "", TQSL_ADIF_RANGE_TYPE_NONE, 10, 0, 0, NULL },
+        { "TIME_ON", "", TQSL_ADIF_RANGE_TYPE_NONE, 10, 0, 0, NULL },
+        { "FREQ", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_FREQ_MAX, 0, 0, NULL },
+        { "FREQ_RX", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_FREQ_MAX, 0, 0, NULL },
+        { "BAND_RX", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_BAND_MAX, 0, 0, NULL },
+        { "SAT_NAME", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_SATNAME_MAX, 0, 0, NULL },
+        { "PROP_MODE", "", TQSL_ADIF_RANGE_TYPE_NONE, TQSL_PROPMODE_MAX, 0, 0, NULL },
+        { "eor", "", TQSL_ADIF_RANGE_TYPE_NONE, 0, 0, 0, NULL },
 };
 
 DLLEXPORT int CALLCONVENTION
@@ -268,16 +268,23 @@ tqsl_endConverter(tQSL_Converter *convp) {
 
 	if ((conv = check_conv(*convp))) {
 		if (conv->txn) conv->txn->abort(conv->txn);
-		if (conv->seendb) conv->seendb->close(conv->seendb, 0);
-		if (conv->dbenv) conv->dbenv->close(conv->dbenv, 0);
+		if (conv->seendb) {
+			conv->seendb->compact(conv->seendb, NULL, NULL, NULL, NULL, 0, NULL);
+			conv->seendb->close(conv->seendb, 0);
+		}
+		if (conv->dbenv) {
+			char **unused;
+			conv->dbenv->txn_checkpoint(conv->dbenv, 0, 0, 0);
+			conv->dbenv->log_archive(conv->dbenv, &unused, DB_ARCH_REMOVE);
+			conv->dbenv->close(conv->dbenv, 0);
+		}
 		// close files and clean up converters, if any
 		if (conv->adif) tqsl_endADIF(&conv->adif);
 		if (conv->cab) tqsl_endCabrillo(&conv->cab);
-		if (conv->cursor) conv->cursor->c_close(conv->cursor);
 		if (conv->dbpath) free(conv->dbpath);
 		if (conv->errfile) fclose(conv->errfile);
 	}
-	
+
 	if (conv->appName) free(conv->appName);
 	if (CAST_TQSL_CONVERTER(*convp)->sentinel == 0x4445)
 		delete CAST_TQSL_CONVERTER(*convp);
@@ -309,7 +316,7 @@ find_matching_cert(TQSL_CONVERTER *conv) {
 	return -1;
 }
 
-static const char *notypes[] = { "D","T","M","N","C","" };
+static const char *notypes[] = { "D", "T", "M", "N", "C", "" };
 
 static const char *
 tqsl_infer_band(const char* infreq) {
@@ -360,24 +367,45 @@ tqsl_setADIFConverterDateFilter(tQSL_Converter convp, tQSL_Date *start, tQSL_Dat
 	return 0;
 }
 
+// Remove the dupes db files
+static void
+remove_db(const char *path)  {
+	DIR *dir = opendir(path);
+	if (dir != NULL) {
+		struct dirent *ent;
+		while ((ent = readdir(dir)) != NULL) {
+			if (!strcmp(ent->d_name, "duplicates.db") ||
+	    		!strncmp(ent->d_name, "log.", 4) ||
+	    		!strncmp(ent->d_name, "__db.", 5)) {
+				string fname = path;
+				fname = fname + "/" + ent->d_name;
+				unlink(fname.c_str());
+			}
+		}
+		closedir(dir);
+	}
+	return;
+}
+//
 // Open the duplicates database
 
-static bool open_db(TQSL_CONVERTER *conv) {
-	bool dbinit_cleanup=false;
+static bool open_db(TQSL_CONVERTER *conv, bool readonly) {
+	bool dbinit_cleanup = false;
 	int dbret;
-	bool triedRemove=false;
-	string fixedpath=tQSL_BaseDir; //must be first because of gotos
-	size_t found=fixedpath.find('\\');
+	bool triedRemove = false;
+	int envflags = DB_INIT_TXN|DB_INIT_LOG|DB_INIT_MPOOL|DB_RECOVER|DB_REGISTER|DB_CREATE;
+	string fixedpath = tQSL_BaseDir; //must be first because of gotos
+	size_t found = fixedpath.find('\\');
 
-	//bdb complains about \\s in path on windows... 
+	//bdb complains about \\s in path on windows...
 
-	while (found!=string::npos) {
+	while (found != string::npos) {
 		fixedpath.replace(found, 1, "/");
-		found=fixedpath.find('\\');
+		found = fixedpath.find('\\');
 	}
 
 	conv->dbpath = strdup(fixedpath.c_str());
-		
+
 #ifndef _WIN32
 	// Clean up junk in that directory
 	DIR *dir = opendir(fixedpath.c_str());
@@ -395,37 +423,62 @@ static bool open_db(TQSL_CONVERTER *conv) {
 				}
 			}
 		}
+		closedir(dir);
 	}
 #endif
-	fixedpath += "/dberr.log";
-	conv->errfile = fopen(fixedpath.c_str(), "wb");
+	string logpath = fixedpath + "/dberr.log";
+	conv->errfile = fopen(logpath.c_str(), "wb");
+
+ reopen:
 
 	while (true) {
 		// Create the database environment handle
 		if ((dbret = db_env_create(&conv->dbenv, 0))) {
 			// can't make env handle
-			dbinit_cleanup=true;
+			dbinit_cleanup = true;
 			goto dbinit_end;
 		}
-		if (conv->errfile)
+		if (conv->errfile) {
 			conv->dbenv->set_errfile(conv->dbenv, conv->errfile);
-		if ((dbret = conv->dbenv->open(conv->dbenv, conv->dbpath, DB_INIT_TXN|DB_INIT_LOCK|DB_INIT_LOG|DB_INIT_MPOOL|DB_CREATE|DB_RECOVER, 0600))) {
+			conv->dbenv->set_verbose(conv->dbenv, DB_VERB_RECOVERY, 1);
+		}
+		// Log files default to 10 Mb each. We don't need nearly that much.
+		if (conv->dbenv->set_lg_max)
+			conv->dbenv->set_lg_max(conv->dbenv, 256 * 1024);
+		// Allocate additional locking resources - some have run out with
+		// the default 1000 locks
+		if (conv->dbenv->set_lk_max_locks)
+			conv->dbenv->set_lk_max_locks(conv->dbenv, 20000);
+		if (conv->dbenv->set_lk_max_objects)
+			conv->dbenv->set_lk_max_objects(conv->dbenv, 20000);
+		if ((dbret = conv->dbenv->open(conv->dbenv, conv->dbpath, envflags, 0600))) {
 			if (conv->errfile)
-				fprintf(conv->errfile, "opening DB %s returns status %d", conv->dbpath, dbret);
+				fprintf(conv->errfile, "opening DB %s returns status %d\n", conv->dbpath, dbret);
 			// can't open environment - try to delete it and try again.
 			if (!triedRemove) {
-				conv->dbenv->remove(conv->dbenv, conv->dbpath, DB_FORCE);
+				conv->dbenv->close(conv->dbenv, 0);
+				// Create a new environment to remove the dross
+				if (!(dbret = db_env_create(&conv->dbenv, 0))) {
+					dbret = conv->dbenv->remove(conv->dbenv, conv->dbpath, DB_FORCE);
+				}
 				triedRemove = true;
 				if (conv->errfile)
 					fprintf(conv->errfile, "About to retry after removing the environment\n");
 				continue;
 			}
+			if (conv->errfile) {
+				fprintf(conv->errfile, "Retry attempt after removing the environment failed.\n");
+			}
+			if (errno == EINVAL) {  // Something really wrong with the DB
+						// Remove it and try again.
+				remove_db(fixedpath.c_str());
+				continue;
+			}
 
-			if (conv->errfile)
-				fprintf(conv->errfile, "Retry attempt after removing the environment failed.");
 			// can't open environment and cleanup efforts failed.
+			conv->dbenv->close(conv->dbenv, 0);
 			conv->dbenv = NULL;	// this can't be recovered
-			dbinit_cleanup=true;
+			dbinit_cleanup = true;
 			goto dbinit_end;
 		}
 		break;		// Opened OK.
@@ -433,30 +486,140 @@ static bool open_db(TQSL_CONVERTER *conv) {
 
 	if ((dbret = db_create(&conv->seendb, conv->dbenv, 0))) {
 		// can't create db
-		dbinit_cleanup=true;
+		dbinit_cleanup = true;
 		goto dbinit_end;
 	}
 
 #ifndef DB_TXN_BULK
 #define DB_TXN_BULK 0
 #endif
-	if ((dbret = conv->dbenv->txn_begin(conv->dbenv, NULL, &conv->txn, DB_TXN_BULK))) {
+	if (!readonly && (dbret = conv->dbenv->txn_begin(conv->dbenv, NULL, &conv->txn, DB_TXN_BULK))) {
 		// can't start a txn
-		dbinit_cleanup=true;
+		dbinit_cleanup = true;
 		goto dbinit_end;
 	}
 
-	if ((dbret = conv->seendb->open(conv->seendb, conv->txn, "duplicates.db", NULL, DB_BTREE, DB_CREATE, 0600))) {
-		// can't open the db
-		dbinit_cleanup=true;
-		goto dbinit_end;
+	// Probe the database type
+	if ((dbret = conv->seendb->open(conv->seendb, conv->txn, "duplicates.db", NULL, DB_UNKNOWN, 0, 0600))) {
+		if (dbret == ENOENT) {
+			dbret = conv->seendb->open(conv->seendb, conv->txn, "duplicates.db", NULL, DB_HASH, DB_CREATE, 0600);
+		}
+		if (dbret) {
+			// can't open the db
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
 	}
 
-dbinit_end:
+	DBTYPE type;
+	conv->seendb->get_type(conv->seendb, &type);
+	if (type ==  DB_BTREE) {
+		// Have to convert the database.
+		string dumpfile = fixedpath + "/dupedump.txt";
+		FILE *dmp = fopen(dumpfile.c_str(), "wb+");
+		if (!dmp) {
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
+		if (!conv->cursor) {
+			int err = conv->seendb->cursor(conv->seendb, conv->txn, &conv->cursor, DB_CURSOR_BULK);
+			if (err) {
+				strncpy(tQSL_CustomError, db_strerror(err), sizeof tQSL_CustomError);
+				tQSL_Error = TQSL_DB_ERROR;
+				tQSL_Errno = errno;
+				dbinit_cleanup = true;
+				goto dbinit_end;
+			}
+		}
+
+		DBT dbkey, dbdata;
+		char duprec[512];
+		while (1) {
+			memset(&dbkey, 0, sizeof dbkey);
+			memset(&dbdata, 0, sizeof dbdata);
+			int status = conv->cursor->c_get(conv->cursor, &dbkey, &dbdata, DB_NEXT);
+			if (DB_NOTFOUND == status) {
+				break;	// No more records
+			}
+			if (status != 0) {
+				strncpy(tQSL_CustomError, db_strerror(status), sizeof tQSL_CustomError);
+				tQSL_Error = TQSL_DB_ERROR;
+				tQSL_Errno = errno;
+				dbinit_cleanup = true;
+				goto dbinit_end;
+			}
+			memcpy(duprec, dbkey.data, dbkey.size);
+			duprec[dbkey.size] = '\0';
+			fprintf(dmp, "%s\n", duprec);
+		}
+		conv->cursor->close(conv->cursor);
+		if (conv->txn) conv->txn->commit(conv->txn, 0);
+		conv->seendb->close(conv->seendb, 0);
+		conv->dbenv->remove(conv->dbenv, conv->dbpath, DB_FORCE);
+		conv->dbenv->close(conv->dbenv, 0);
+		conv->cursor = NULL;
+		conv->seendb = NULL;
+		conv->dbenv = NULL;
+
+		// Remove the old dupe db
+		remove_db(fixedpath.c_str());
+
+		// Now create the new database
+		if ((dbret = db_env_create(&conv->dbenv, 0))) {
+			// can't make env handle
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
+		if (conv->errfile)
+			conv->dbenv->set_errfile(conv->dbenv, conv->errfile);
+		if (conv->dbenv->set_lg_max)
+			conv->dbenv->set_lg_max(conv->dbenv, 256 * 1024);
+		if (conv->dbenv->set_lk_max_locks)
+			conv->dbenv->set_lk_max_locks(conv->dbenv, 20000);
+		if (conv->dbenv->set_lk_max_objects)
+			conv->dbenv->set_lk_max_objects(conv->dbenv, 20000);
+		if ((dbret = conv->dbenv->open(conv->dbenv, conv->dbpath, envflags, 0600))) {
+			if (conv->errfile)
+				fprintf(conv->errfile, "opening DB %s returns status %d\n", conv->dbpath, dbret);
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
+
+		if ((dbret = db_create(&conv->seendb, conv->dbenv, 0))) {
+			// can't create db
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
+
+		// Create the new database
+		if ((dbret = conv->seendb->open(conv->seendb, NULL, "duplicates.db", NULL, DB_HASH, DB_CREATE, 0600))) {
+			// can't open the db
+			dbinit_cleanup = true;
+			goto dbinit_end;
+		}
+		fseek(dmp, 0, SEEK_SET);
+
+		char d[1]= {'D'};
+		memset(&dbkey, 0, sizeof dbkey);
+		memset(&dbdata, 0, sizeof dbdata);
+		dbdata.data = d;
+		dbdata.size = 1;
+
+		while (fgets(duprec, sizeof duprec, dmp)) {
+			dbkey.data = duprec;
+			dbkey.size = strlen(duprec) - 1;
+			conv->seendb->put(conv->seendb, NULL, &dbkey, &dbdata, 0);
+		}
+		conv->seendb->close(conv->seendb, 0);
+		conv->dbenv->close(conv->dbenv, 0);
+		goto reopen;
+	}
+
+ dbinit_end:
 	if (dbinit_cleanup) {
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
-		strcpy(tQSL_CustomError, db_strerror(dbret));
+		strncpy(tQSL_CustomError, db_strerror(dbret), sizeof tQSL_CustomError);
 		if (conv->txn) conv->txn->abort(conv->txn);
 		if (conv->seendb) conv->seendb->close(conv->seendb, 0);
 		if (conv->dbenv) {
@@ -467,7 +630,7 @@ dbinit_end:
 			}
 			conv->dbenv->close(conv->dbenv, 0);
 		}
-		if (conv->cursor) conv->cursor->c_close(conv->cursor);
+		if (conv->cursor) conv->cursor->close(conv->cursor);
 		if (conv->errfile) fclose(conv->errfile);
 		conv->txn = NULL;
 		conv->dbenv = NULL;
@@ -497,7 +660,8 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
                         major, minor, config_major, config_minor,
                         conv->allow_dupes ? "true" : "false");
 		temp[sizeof temp - 1] = '\0';
-		snprintf(ident, sizeof ident, "<TQSL_IDENT:%d>%s\n", (int)strlen(temp), temp);
+		int len = strlen(temp);
+		snprintf(ident, sizeof ident, "<TQSL_IDENT:%d>%s\n", len, temp);
 		ident[sizeof ident - 1] = '\0';
 		conv->need_ident_rec = false;
 		return ident;
@@ -511,7 +675,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		return tStation;
 	}
 	if (!conv->allow_dupes && !conv->seendb) {
-		if (!open_db(conv)) {	// If can't open dupes DB
+		if (!open_db(conv, false)) {	// If can't open dupes DB
 			return 0;
 		}
 	}
@@ -526,7 +690,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		int saveErr = 0;
 		if (conv->adif) {
 	 		while (1) {
- 				tqsl_adifFieldResults result;
+				tqsl_adifFieldResults result;
 				if (tqsl_getADIFField(conv->adif, &result, &stat, adif_qso_record_fields, notypes, adif_allocate))
 					break;
 				if (stat != TQSL_ADIF_GET_FIELD_SUCCESS && stat != TQSL_ADIF_GET_FIELD_NO_NAME_MATCH)
@@ -534,40 +698,42 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				if (!strcasecmp(result.name, "eor"))
 					break;
 				if (!strcasecmp(result.name, "CALL") && result.data) {
-					strncpy(conv->rec.callsign, (char *)(result.data), sizeof conv->rec.callsign);
+					strncpy(conv->rec.callsign, reinterpret_cast<char *>(result.data), sizeof conv->rec.callsign);
 				} else if (!strcasecmp(result.name, "BAND") && result.data) {
-					strncpy(conv->rec.band, (char *)(result.data), sizeof conv->rec.band);
+					strncpy(conv->rec.band, reinterpret_cast<char *>(result.data), sizeof conv->rec.band);
 				} else if (!strcasecmp(result.name, "MODE") && result.data) {
-					strncpy(conv->rec.mode, (char *)(result.data), sizeof conv->rec.mode);
+					strncpy(conv->rec.mode, reinterpret_cast<char *>(result.data), sizeof conv->rec.mode);
 				} else if (!strcasecmp(result.name, "FREQ") && result.data) {
-					strncpy(conv->rec.freq, (char *)(result.data), sizeof conv->rec.freq);
+					strncpy(conv->rec.freq, reinterpret_cast<char *>(result.data), sizeof conv->rec.freq);
 					if (atof(conv->rec.freq) == 0.0)
 						conv->rec.freq[0] = '\0';
 				} else if (!strcasecmp(result.name, "FREQ_RX") && result.data) {
-					strncpy(conv->rec.rxfreq, (char *)(result.data), sizeof conv->rec.rxfreq);
+					strncpy(conv->rec.rxfreq, reinterpret_cast<char *>(result.data), sizeof conv->rec.rxfreq);
 					if (atof(conv->rec.rxfreq) == 0.0)
 						conv->rec.rxfreq[0] = '\0';
 				} else if (!strcasecmp(result.name, "BAND_RX") && result.data) {
-					strncpy(conv->rec.rxband, (char *)(result.data), sizeof conv->rec.rxband);
+					strncpy(conv->rec.rxband, reinterpret_cast<char *>(result.data), sizeof conv->rec.rxband);
 				} else if (!strcasecmp(result.name, "SAT_NAME") && result.data) {
-					strncpy(conv->rec.satname, (char *)(result.data), sizeof conv->rec.satname);
+					strncpy(conv->rec.satname, reinterpret_cast<char *>(result.data), sizeof conv->rec.satname);
 				} else if (!strcasecmp(result.name, "PROP_MODE") && result.data) {
-					strncpy(conv->rec.propmode, (char *)(result.data), sizeof conv->rec.propmode);
+					strncpy(conv->rec.propmode, reinterpret_cast<char *>(result.data), sizeof conv->rec.propmode);
 				} else if (!strcasecmp(result.name, "QSO_DATE") && result.data) {
 					cstat = tqsl_initDate(&(conv->rec.date), (const char *)result.data);
+					if (cstat)
+						saveErr = tQSL_Error;
 				} else if (!strcasecmp(result.name, "TIME_ON") && result.data) {
 					cstat = tqsl_initTime(&(conv->rec.time), (const char *)result.data);
+					if (cstat)
+						saveErr = tQSL_Error;
 				}
 				if (stat == TQSL_ADIF_GET_FIELD_SUCCESS) {
-					conv->rec_text += string((char *)result.name) + ": ";
+					conv->rec_text += string(reinterpret_cast<char *>(result.name)) + ": ";
 					if (result.data)
-						conv->rec_text += string((char *)result.data);
+						conv->rec_text += string(reinterpret_cast<char *>(result.data));
 					conv->rec_text += "\n";
 				}
 				if (result.data)
-						delete[] result.data;
-				if (cstat)
-				    saveErr = tQSL_Error;
+					delete[] result.data;
 			}
 			if (saveErr) {
 				tQSL_Error = saveErr;
@@ -604,14 +770,16 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 						strncpy(conv->rec.freq, field.value, sizeof conv->rec.freq);
 					} else if (!strcasecmp(field.name, "QSO_DATE")) {
 						cstat = tqsl_initDate(&(conv->rec.date), field.value);
+						if (cstat)
+							saveErr = tQSL_Error;
 					} else if (!strcasecmp(field.name, "TIME_ON")) {
 						cstat = tqsl_initTime(&(conv->rec.time), field.value);
+						if (cstat)
+							saveErr = tQSL_Error;
 					}
 					if (conv->rec_text != "")
 						conv->rec_text += "\n";
 					conv->rec_text += string(field.name) + ": " + field.value;
-					if (cstat)
-						saveErr = tQSL_Error;
 				}
 			} while (stat == TQSL_CABRILLO_NO_ERROR);
 			if (saveErr)
@@ -622,7 +790,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			}
 		} else {
 			tQSL_Error = TQSL_CUSTOM_ERROR;
-			strcpy(tQSL_CustomError, "Converter not initialized");
+			strncpy(tQSL_CustomError, "Converter not initialized", sizeof tQSL_CustomError);
 			return 0;
 		}
 	}
@@ -752,8 +920,6 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			strncat(signdata, conv->serial, sizeof(signdata) - strlen(signdata)-1);
 			dbkey.size = strlen(signdata);
 			dbkey.data = signdata;
-			dbdata.size = sizeof(temp);
-			dbdata.data = temp;
 			int dbget_err = conv->seendb->get(conv->seendb, conv->txn, &dbkey, &dbdata, 0);
 			if (0 == dbget_err) {
 				//lookup was successful; thus duplicate
@@ -761,21 +927,21 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				return 0;
 			} else if (dbget_err != DB_NOTFOUND) {
 				//non-zero return, but not "not found" - thus error
-				strcpy(tQSL_CustomError, db_strerror(dbget_err));
+				strncpy(tQSL_CustomError, db_strerror(dbget_err), sizeof tQSL_CustomError);
 				tQSL_Error = TQSL_DB_ERROR;
 				return 0;
 				// could be more specific but there's very little the user can do at this point anyway
 			}
 			temp[0] = 'D';
+			dbdata.data = temp;
 			dbdata.size = 1;
 			int dbput_err;
 			dbput_err = conv->seendb->put(conv->seendb, conv->txn, &dbkey, &dbdata, 0);
 			if (0 != dbput_err) {
-				strcpy(tQSL_CustomError, db_strerror(dbput_err));
+				strncpy(tQSL_CustomError, db_strerror(dbput_err), sizeof tQSL_CustomError);
 				tQSL_Error = TQSL_DB_ERROR;
 				return 0;
 			}
-			
 		}
 	}
 	return grec;
@@ -857,7 +1023,7 @@ tqsl_converterRollBack(tQSL_Converter convp) {
 	if (!(conv = check_conv(convp)))
 		return 1;
 	if (!conv->seendb)
-		return 1;
+		return 0;
 	if (conv->txn)
 		conv->txn->abort(conv->txn);
 	conv->txn = NULL;
@@ -871,10 +1037,10 @@ tqsl_converterCommit(tQSL_Converter convp) {
 	if (!(conv = check_conv(convp)))
 		return 1;
 	if (!conv->seendb)
-		return 1;
+		return 0;
 	if (conv->txn)
 		conv->txn->commit(conv->txn, 0);
-	conv->txn=NULL;
+	conv->txn = NULL;
 	return 0;
 }
 
@@ -886,7 +1052,7 @@ tqsl_getDuplicateRecords(tQSL_Converter convp, char *key, char *data, int keylen
 		return 1;
 
 	if (!conv->seendb) {
-		if (!open_db(conv)) {	// If can't open dupes DB
+		if (!open_db(conv, true)) {	// If can't open dupes DB
 			return 1;
 		}
 	}
@@ -896,7 +1062,7 @@ tqsl_getDuplicateRecords(tQSL_Converter convp, char *key, char *data, int keylen
 	if (!conv->cursor) {
 		int err = conv->seendb->cursor(conv->seendb, conv->txn, &conv->cursor, DB_CURSOR_BULK);
 		if (err) {
-			strcpy(tQSL_CustomError, db_strerror(err));
+			strncpy(tQSL_CustomError, db_strerror(err), sizeof tQSL_CustomError);
 			tQSL_Error = TQSL_DB_ERROR;
 			tQSL_Errno = errno;
 			return 1;
@@ -911,7 +1077,7 @@ tqsl_getDuplicateRecords(tQSL_Converter convp, char *key, char *data, int keylen
 		return -1;	// No more records
 	}
 	if (status != 0) {
-		strcpy(tQSL_CustomError, db_strerror(status));
+		strncpy(tQSL_CustomError, db_strerror(status), sizeof tQSL_CustomError);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
@@ -931,7 +1097,7 @@ tqsl_putDuplicateRecord(tQSL_Converter convp, const char *key, const char *data,
 		return 0;
 
 	if (!conv->seendb) {
-		if (!open_db(conv)) {	// If can't open dupes DB
+		if (!open_db(conv, false)) {	// If can't open dupes DB
 			return 0;
 		}
 	}
@@ -939,9 +1105,10 @@ tqsl_putDuplicateRecord(tQSL_Converter convp, const char *key, const char *data,
 	memset(&dbkey, 0, sizeof dbkey);
 	memset(&dbdata, 0, sizeof dbdata);
 	dbkey.size = keylen;
-	dbkey.data = (char *)key;
-	dbdata.size = 2;
-	dbdata.data = (char *)data;
+	dbkey.data = const_cast<char *>(key);
+
+	dbdata.size = 1;
+	dbdata.data = const_cast<char *>(data);
 
 	int status = conv->seendb->put(conv->seendb, conv->txn, &dbkey, &dbdata, 0);
 
@@ -950,7 +1117,7 @@ tqsl_putDuplicateRecord(tQSL_Converter convp, const char *key, const char *data,
 	}
 
 	if (status != 0) {
-		strcpy(tQSL_CustomError, db_strerror(status));
+		strncpy(tQSL_CustomError, db_strerror(status), sizeof tQSL_CustomError);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
@@ -969,6 +1136,17 @@ hasValidCallSignChars(const string& call) {
 	// Need at least one number
 	if (call.find_first_of("0123456789") == string::npos)
 		return false;
+	// Invalid callsign patterns
+	// Starting with 0, Q, C7, or 4Y
+	// 1x other than 1A, 1M, 1S
+	string first = call.substr(0, 1);
+	string second = call.substr(1, 1);
+	if (first == "0" || first == "Q" ||
+	    (first == "C" && second == "7") ||
+	    (first == "4" && second == "Y") ||
+	    (first == "1" && second != "A" && second != "M" && second != "S"))
+		return false;
+
 	return true;
 }
 
